#ifndef HV_H_SEEN
#define HV_H_SEEN

#include <termios.h>		// POSIX terminal control definitions
#include <fcntl.h>		// File control definitions => fcntl() in fcn OpenPort()
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <string>
#include <vector>
#include <map>

#include "dis.hxx"

#define MAX_NUM_BOARDS 13	// Maximum number of boards per crate
#define NUM_CHANNELS 32		// Channels per bias board
#define BAUDRATE B115200

class User;

class Crate {
  
    class User *m;
	int CrateNumber;
	int fDescriptor;
	FILE *File;
	DimService *NameService;

	std::vector<char> Communicate(std::string);
	void ClearVoltageArrays();
   
  public:
    Crate(std::string, int, class User *);
    ~Crate();

    char *Name;
	DimService *BiasVolt;

	int Current[MAX_NUM_BOARDS][NUM_CHANNELS];
	bool OC[MAX_NUM_BOARDS][NUM_CHANNELS];
	bool Present[MAX_NUM_BOARDS][NUM_CHANNELS];
	bool ResetHit;
	bool WrapOK;
	int WrapCount;
	int ErrorCount;

	unsigned int DAC[MAX_NUM_BOARDS][NUM_CHANNELS];      // Voltage in DAC units
	double Volt[MAX_NUM_BOARDS][NUM_CHANNELS];  // Voltage in Volt
	int CurrentOffset[MAX_NUM_BOARDS][NUM_CHANNELS]; // Offset for current measurement

	bool InitOK;

	int SetChannels(std::map<unsigned int, unsigned int>);
	int ReadAll();
	int SystemReset();
	int GlobalSet(unsigned int);
	bool Synch();
	bool CurrentCalib(double);
};

#endif
